@@ -6,7 +6,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sndio.h>
-#include <pthread.h>
 #include <re.h>
 #include <rem.h>
 #include <baresip.h>
@@ -21,22 +20,24 @@
 
 struct ausrc_st {
 	struct sio_hdl *hdl;
-	pthread_t thread;
+	thrd_t thread;
 	int16_t *sampv;
 	size_t sampc;
 	int run;
 	ausrc_read_h *rh;
 	void *arg;
+	struct ausrc_prm prm;
 };
 
 struct auplay_st {
 	struct sio_hdl *hdl;
-	pthread_t thread;
+	thrd_t thread;
 	int16_t *sampv;
 	size_t sampc;
 	int run;
 	auplay_write_h *wh;
 	void *arg;
+	struct auplay_prm prm;
 };
 
 static struct ausrc *ausrc;
@@ -66,7 +67,7 @@ static struct sio_par *sndio_initpar(uint32_t srate, uint8_t ch)
 }
 
 
-static void *read_thread(void *arg)
+static int read_thread(void *arg)
 {
 	struct ausrc_st *st = arg;
 
@@ -79,19 +80,18 @@ static void *read_thread(void *arg)
 		struct auframe af;
 		size_t n = sio_read(st->hdl, st->sampv, st->sampc*2);
 
-		af.fmt   = AUFMT_S16LE;
-		af.sampv = st->sampv;
-		af.sampc = n/2;
+		auframe_init(&af, AUFMT_S16LE, st->sampv, n/2,
+			     st->prm.srate, st->prm.ch);
 
 		st->rh(&af, st->arg);
 	}
 
  out:
-	return NULL;
+	return 0;
 }
 
 
-static void *write_thread(void *arg)
+static int write_thread(void *arg)
 {
 	struct auplay_st *st = arg;
 	struct auframe af;
@@ -101,7 +101,8 @@ static void *write_thread(void *arg)
 		goto out;
 	}
 
-	auframe_init(&af, AUFMT_S16LE, st->sampv, st->sampc);
+	auframe_init(&af, AUFMT_S16LE, st->sampv, st->sampc, st->prm.srate,
+		     st->prm.ch);
 
 	while (st->run) {
 		st->wh(&af, st->arg);
@@ -110,7 +111,7 @@ static void *write_thread(void *arg)
 	}
 
  out:
-	return NULL;
+	return 0;
 }
 
 
@@ -120,7 +121,7 @@ static void ausrc_destructor(void *arg)
 
 	if (st->run) {
 		st->run = false;
-		(void)pthread_join(st->thread, NULL);
+		(void)thrd_join(st->thread, NULL);
 	}
 
 	if (st->hdl)
@@ -136,7 +137,7 @@ static void auplay_destructor(void *arg)
 
 	if (st->run) {
 		st->run = false;
-		(void)pthread_join(st->thread, NULL);
+		(void)thrd_join(st->thread, NULL);
 	}
 
 	if (st->hdl)
@@ -147,7 +148,6 @@ static void auplay_destructor(void *arg)
 
 
 static int src_alloc(struct ausrc_st **stp, const struct ausrc *as,
-		     struct media_ctx **ctx,
 		     struct ausrc_prm *prm, const char *device,
 		     ausrc_read_h *rh, ausrc_error_h *errh, void *arg)
 {
@@ -156,7 +156,6 @@ static int src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 	int err;
 	const char *name;
 
-	(void)ctx;
 	(void)errh;
 
 	if (!stp || !as || !prm)
@@ -175,6 +174,7 @@ static int src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 
 	st->rh  = rh;
 	st->arg = arg;
+	st->prm = *prm;
 	st->hdl = sio_open(name, SIO_REC, 0);
 
 	if (!st->hdl) {
@@ -208,7 +208,7 @@ static int src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 	}
 
 	st->run = true;
-	err = pthread_create(&st->thread, NULL, read_thread, st);
+	err = thread_create_name(&st->thread, "sndio_read", read_thread, st);
 	if (err)
 		st->run = false;
 
@@ -248,6 +248,7 @@ static int play_alloc(struct auplay_st **stp, const struct auplay *ap,
 
 	st->wh  = wh;
 	st->arg = arg;
+	st->prm = *prm;
 	st->hdl = sio_open(name, SIO_PLAY, 0);
 
 	if (!st->hdl) {
@@ -281,7 +282,7 @@ static int play_alloc(struct auplay_st **stp, const struct auplay *ap,
 	}
 
 	st->run = true;
-	err = pthread_create(&st->thread, NULL, write_thread, st);
+	err = thread_create_name(&st->thread, "sndio_write", write_thread, st);
 	if (err)
 		st->run = false;
 
