@@ -3,12 +3,14 @@
  *
  * Copyright (C) 2010 - 2015 Alfred E. Heggestad
  */
+#include <stdint.h>
 #include <string.h>
 #include <stdlib.h>
 #include <re.h>
 #include <rem.h>
 #include <baresip.h>
 #include "test.h"
+#include "call.h"
 #include "sip/sipsrv.h"
 #include "../src/core.h"  /* NOTE: temp */
 
@@ -18,930 +20,6 @@
 #include <re_dbg.h>
 
 
-#define MAGIC 0x7004ca11
-
-enum {
-	IP_127_0_0_1 = 0x7f000001,
-};
-
-enum behaviour {
-	BEHAVIOUR_ANSWER = 0,
-	BEHAVIOUR_NOTHING,
-	BEHAVIOUR_REJECT,
-	BEHAVIOUR_REJECTF,
-	BEHAVIOUR_GET_HDRS,
-	BEHAVIOUR_PROGRESS,
-};
-
-enum action {
-	ACTION_RECANCEL = 0,
-	ACTION_HANGUP_A,
-	ACTION_HANGUP_B,
-	ACTION_NOTHING,
-	ACTION_TRANSFER,
-	ACTION_ATT_TRANSFER,
-};
-
-
-struct cancel_rule {
-	struct le le;
-
-	enum bevent_ev ev;
-	const char *prm;
-	struct ua *ua;
-	bool checkack;
-
-	unsigned n_incoming;
-	unsigned n_progress;
-	unsigned n_established;
-	unsigned n_audio_estab;
-	unsigned n_video_estab;
-	unsigned n_offer_cnt;
-	unsigned n_answer_cnt;
-	unsigned n_vidframe;
-	unsigned n_auframe;
-	unsigned n_audebug;
-	unsigned n_rtcp;
-	unsigned n_closed;
-	double aulvl;
-
-	struct cancel_rule *cr_and;
-	bool met;
-};
-
-
-struct agent {
-	struct fixture *fix;    /* pointer to parent */
-	struct agent *peer;
-	struct ua *ua;
-	uint16_t close_scode;
-	char *close_prm;
-	bool failed;
-
-	unsigned n_incoming;
-	unsigned n_progress;
-	unsigned n_established;
-	unsigned n_closed;
-	unsigned n_transfer_fail;
-	unsigned n_dtmf_recv;
-	unsigned n_transfer;
-	unsigned n_mediaenc;
-	unsigned n_rtpestab;
-	unsigned n_rtcp;
-	unsigned n_audio_estab;
-	unsigned n_video_estab;
-	unsigned n_offer_cnt;
-	unsigned n_answer_cnt;
-	unsigned n_hold_cnt;
-	unsigned n_resume_cnt;
-	unsigned n_vidframe;
-	unsigned n_auframe;
-	unsigned n_audebug;
-	double aulvl;
-
-	struct tmr tmr_ack;
-	bool gotack;
-
-	struct tmr tmr;
-};
-
-
-struct fixture {
-	uint32_t magic;
-	struct agent a, b, c;
-	struct sa dst;
-	struct sa laddr_udp;
-	struct sa laddr_tcp;
-	struct sa laddr_tls;
-	enum behaviour behaviour;
-	enum action estab_action;
-	char buri[256];
-	char buri_tcp[256];
-	int err;
-	struct call *xfer;
-	unsigned exp_estab;
-	unsigned exp_closed;
-	bool fail_transfer;
-	struct list rules;
-};
-
-
-#define fixture_init_prm(f, prm)					\
-	memset(f, 0, sizeof(*f));					\
-									\
-	f->a.fix = f;							\
-	f->b.fix = f;							\
-	f->c.fix = f;							\
-	err = sa_set_str(&f->dst, "127.0.0.1", 5060);			\
-	TEST_ERR(err);							\
-									\
-	err = ua_init("test", true, true, true);			\
-	TEST_ERR(err);							\
-									\
-	f->magic = MAGIC;						\
-	f->estab_action = ACTION_RECANCEL;				\
-	f->exp_estab = 1;						\
-	f->exp_closed = 1;						\
-	/* NOTE: See Makefile TEST_MODULES */				\
-	err = module_load(".", "g711");					\
-	TEST_ERR(err);							\
-									\
-	err = ua_alloc(&f->a.ua,					\
-		       "A <sip:a@127.0.0.1>;regint=0" prm);		\
-	TEST_ERR(err);							\
-	err = ua_alloc(&f->b.ua,					\
-		       "B <sip:b@127.0.0.1>;regint=0" prm);		\
-	TEST_ERR(err);							\
-									\
-	f->a.peer = &f->b;						\
-	f->b.peer = &f->a;						\
-									\
-	err = bevent_register(event_handler, f);			\
-	TEST_ERR(err);							\
-									\
-	err = sip_transp_laddr(uag_sip(), &f->laddr_udp,		\
-			       SIP_TRANSP_UDP, &f->dst);		\
-	TEST_ERR(err);							\
-									\
-	err = sip_transp_laddr(uag_sip(), &f->laddr_tcp,		\
-			       SIP_TRANSP_TCP, &f->dst);		\
-	TEST_ERR(err);							\
-									\
-	err = sip_transp_laddr(uag_sip(), &f->laddr_tls,		\
-			       SIP_TRANSP_TLS, &f->dst);		\
-	TEST_ERR(err);							\
-									\
-	debug("test: local SIP transp: UDP=%J, TCP=%J\n",		\
-	      &f->laddr_udp, &f->laddr_tcp);				\
-									\
-	re_snprintf(f->buri, sizeof(f->buri),				\
-		    "sip:b@%J", &f->laddr_udp);				\
-	re_snprintf(f->buri_tcp, sizeof(f->buri_tcp),			\
-		    "sip:b@%J;transport=tcp", &f->laddr_tcp);
-
-
-#define fixture_init(f)				\
-	fixture_init_prm((f), "");
-
-
-#define fixture_close(f)			\
-	tmr_cancel(&f->a.tmr_ack);		\
-	tmr_cancel(&f->b.tmr_ack);		\
-	tmr_cancel(&f->c.tmr_ack);		\
-	tmr_cancel(&f->a.tmr);			\
-	tmr_cancel(&f->b.tmr);			\
-	tmr_cancel(&f->c.tmr);			\
-	mem_deref(f->c.ua);			\
-	mem_deref(f->b.ua);			\
-	mem_deref(f->a.ua);			\
-	mem_deref(f->c.close_prm);		\
-	mem_deref(f->b.close_prm);		\
-	mem_deref(f->a.close_prm);		\
-						\
-	module_unload("g711");			\
-						\
-	bevent_unregister(event_handler);	\
-						\
-	ua_stop_all(true);			\
-	ua_close();				\
-	list_flush(&f->rules);
-
-#define fixture_abort(f, error)			\
-	do {					\
-		(f)->err = (error);		\
-		re_cancel();			\
-	} while (0)
-
-
-static void cancel_rule_destructor(void *arg)
-{
-	struct cancel_rule *r = arg;
-
-	list_unlink(&r->le);
-	mem_deref(r->cr_and);
-}
-
-
-static struct cancel_rule *cancel_rule_alloc(enum bevent_ev ev,
-					     struct ua *ua,
-					     unsigned n_incoming,
-					     unsigned n_progress,
-					     unsigned n_established)
-{
-	struct cancel_rule *r = mem_zalloc(sizeof(*r), cancel_rule_destructor);
-	if (!r)
-		return NULL;
-
-	r->ev = ev;
-	r->ua = ua;
-	r->n_incoming    = n_incoming;
-	r->n_progress    = n_progress;
-	r->n_established = n_established;
-
-	r->n_audio_estab = (unsigned) -1;
-	r->n_video_estab = (unsigned) -1;
-	r->n_offer_cnt   = (unsigned) -1;
-	r->n_answer_cnt  = (unsigned) -1;
-	r->n_vidframe    = (unsigned) -1;
-	r->n_auframe     = (unsigned) -1;
-	r->n_audebug     = (unsigned) -1;
-	r->n_rtcp        = (unsigned) -1;
-	r->n_closed      = (unsigned) -1;
-	r->aulvl         = 0.0f;
-	return r;
-}
-
-
-static struct cancel_rule *fixture_add_cancel_rule(struct fixture *f,
-						   enum bevent_ev ev,
-						   struct ua *ua,
-						   unsigned n_incoming,
-						   unsigned n_progress,
-						   unsigned n_established)
-{
-	struct cancel_rule *r = cancel_rule_alloc(ev, ua, n_incoming,
-						  n_progress, n_established);
-	if (!r)
-		return NULL;
-
-	list_append(&f->rules, &r->le, r);
-	return r;
-}
-
-
-static struct cancel_rule *cancel_rule_and_alloc(struct cancel_rule *cr,
-						 enum bevent_ev ev,
-						 struct ua *ua,
-						 unsigned n_incoming,
-						 unsigned n_progress,
-						 unsigned n_established)
-{
-	struct cancel_rule *r = cancel_rule_alloc(ev, ua, n_incoming,
-						  n_progress, n_established);
-	if (!r)
-		return NULL;
-
-	cr->cr_and = r;
-	return r;
-}
-
-
-#define UINTSET(u) (u) != (unsigned) -1
-
-#define cr_debug_nbr(fld) \
-	(UINTSET(cr->fld) ? \
-		re_hprintf(pf, "    " #fld ":    %u\n", cr->fld) : 0)
-
-
-static int cancel_rule_debug(struct re_printf *pf,
-			     const struct cancel_rule *cr)
-{
-	int err;
-	if (!cr)
-		return 0;
-
-	err  = re_hprintf(pf, "  --- %s ---\n", bevent_str(cr->ev));
-	err |= re_hprintf(pf, "    prm:  %s\n", cr->prm);
-	err |= re_hprintf(pf, "    ua:   %s\n",
-			  account_aor(ua_account(cr->ua)));
-	err |= cr_debug_nbr(n_incoming);
-	err |= cr_debug_nbr(n_progress);
-	err |= cr_debug_nbr(n_established);
-	err |= cr_debug_nbr(n_audio_estab);
-	err |= cr_debug_nbr(n_video_estab);
-	err |= cr_debug_nbr(n_offer_cnt);
-	err |= cr_debug_nbr(n_answer_cnt);
-	err |= cr_debug_nbr(n_auframe);
-	err |= cr_debug_nbr(n_vidframe);
-	err |= cr_debug_nbr(n_audebug);
-	err |= cr_debug_nbr(n_rtcp);
-	err |= cr_debug_nbr(n_closed);
-	err |= re_hprintf(pf, "    met:  %s\n", cr->met ? "yes": "no");
-	if (err)
-		return err;
-
-	if (cr->cr_and) {
-		err |= re_hprintf(pf, "  AND -->\n");
-		err |= cancel_rule_debug(pf, cr->cr_and);
-	}
-
-	return err;
-}
-
-
-#define ag_debug_nbr(fld) \
-	re_hprintf(pf, "    " #fld ": %u\n", ag->fld)
-
-
-static int agent_debug(struct re_printf *pf, const struct agent *ag)
-{
-	int err;
-	if (!ag)
-		return 0;
-
-	char c = &ag->fix->a == ag ? 'a' :
-		 &ag->fix->b == ag ? 'b' : 'c';
-	err = re_hprintf(pf, "  --- Agent %c ---\n", c);
-	err |= ag_debug_nbr(close_scode);
-	err |= re_hprintf(pf, "    failed: %s\n", ag->failed ? "yes" : "no");
-	err |= ag_debug_nbr(n_incoming);
-	err |= ag_debug_nbr(n_progress);
-	err |= ag_debug_nbr(n_established);
-	err |= ag_debug_nbr(n_closed);
-	err |= ag_debug_nbr(n_transfer_fail);
-	err |= ag_debug_nbr(n_dtmf_recv);
-	err |= ag_debug_nbr(n_transfer);
-	err |= ag_debug_nbr(n_mediaenc);
-	err |= ag_debug_nbr(n_rtpestab);
-	err |= ag_debug_nbr(n_rtcp);
-	err |= ag_debug_nbr(n_audio_estab);
-	err |= ag_debug_nbr(n_video_estab);
-	err |= ag_debug_nbr(n_offer_cnt);
-	err |= ag_debug_nbr(n_answer_cnt);
-	err |= ag_debug_nbr(n_hold_cnt);
-	err |= ag_debug_nbr(n_resume_cnt);
-	err |= ag_debug_nbr(n_auframe);
-	err |= ag_debug_nbr(n_audebug);
-	err |= ag_debug_nbr(n_vidframe);
-
-	return err;
-}
-
-
-static void failure_debug(struct fixture *f, bool c)
-{
-	struct le *le;
-
-	re_printf("Cancel Rules:\n");
-	LIST_FOREACH(&f->rules, le) {
-		struct cancel_rule *cr = le->data;
-
-		re_printf("%H", cancel_rule_debug, cr);
-	}
-
-	re_printf("Agents:\n");
-	re_printf("%H", agent_debug, &f->a);
-	re_printf("%H", agent_debug, &f->b);
-	if (c)
-		re_printf("%H", agent_debug, &f->c);
-}
-
-
-static void cancel_rule_reset(struct cancel_rule *cr)
-{
-	cr->met = false;
-
-	if (cr->cr_and)
-		cancel_rule_reset(cr->cr_and);
-}
-
-
-static void cancel_rules_reset(struct fixture *f)
-{
-	struct le *le;
-
-	LIST_FOREACH(&f->rules, le) {
-		struct cancel_rule *cr = le->data;
-
-		cancel_rule_reset(cr);
-	}
-}
-
-
-#define cancel_rule_new(ev, ua, n_incoming, n_progress, n_established)    \
-	cr = fixture_add_cancel_rule(f, ev, ua, n_incoming, n_progress,   \
-				     n_established);			  \
-	if (!cr) {							  \
-		err = ENOMEM;						  \
-		goto out;						  \
-	}
-
-
-#define cancel_rule_and(ev, ua, n_incoming, n_progress, n_established)	  \
-	cr = cancel_rule_and_alloc(cr, ev, ua, n_incoming, n_progress,	  \
-				   n_established);			  \
-	if (!cr) {							  \
-		err = ENOMEM;						  \
-		goto out;						  \
-	}
-
-
-#define cancel_rule_pop()						  \
-	mem_deref(list_tail(&f->rules)->data);
-
-
-#define UINTSET(u) (u) != (unsigned) -1
-
-static const struct list *hdrs;
-
-
-static const char dtmf_digits[] = "123";
-
-
-static void check_ack(void *arg)
-{
-	struct agent *ag = arg;
-
-	if (ag->gotack)
-		return;
-
-	ag->gotack = !call_ack_pending(ua_call(ag->ua));
-
-	if (ag->gotack)
-		bevent_ua_emit(BEVENT_CUSTOM, ag->ua, "gotack");
-
-	else
-		tmr_start(&ag->tmr_ack, 1, check_ack, ag);
-}
-
-
-static int agent_wait_for_ack(struct agent *ag, unsigned n_incoming,
-			      unsigned n_progress, unsigned n_established)
-{
-	int err;
-	struct cancel_rule *cr;
-	struct fixture *f = ag->fix;
-
-	if (!call_ack_pending(ua_call(ag->ua)))
-		return 0;
-
-	cancel_rule_new(BEVENT_CUSTOM, ag->ua, n_incoming, n_progress,
-			n_established);
-	cr->prm = "gotack";
-	cr->checkack = true;
-
-	ag->gotack = false;
-	tmr_start(&ag->tmr_ack, 1, check_ack, ag);
-	err = re_main_timeout(10000);
-	cancel_rule_pop();
-	if (err)
-		goto out;
-
-	err = call_ack_pending(ua_call(ag->ua)) ? ETIMEDOUT : 0;
-
-out:
-	return err;
-}
-
-
-static bool check_rule(struct cancel_rule *rule, int met_prev,
-		       struct agent *ag, enum bevent_ev ev, const char *prm)
-{
-	bool met_next = true;
-	if (rule->cr_and) {
-		met_next = check_rule(rule->cr_and, rule->met && met_prev, ag,
-				      ev, prm);
-		if (rule->met && met_prev && met_next)
-			return true;
-	}
-
-	if (rule->met)
-		goto out;
-
-	if (ev != rule->ev)
-		return false;
-
-	if (str_isset(rule->prm) &&
-	    !str_str(prm, rule->prm)) {
-		info("test: event %s prm=[%s] (expected [%s])\n",
-		     bevent_str(ev), prm, rule->prm);
-		return false;
-	}
-
-	if (rule->ua &&
-	    ag->ua != rule->ua) {
-		info("test: event %s ua=[%s] (expected [%s]\n",
-		     bevent_str(ev),
-		     account_aor(ua_account(ag->ua)),
-		     account_aor(ua_account(rule->ua)));
-		return false;
-	}
-
-	if (rule->checkack && !ag->gotack) {
-		info("test: event %s waiting for ACK\n", bevent_str(ev));
-		return false;
-	}
-
-	if (UINTSET(rule->n_incoming) &&
-	    ag->n_incoming != rule->n_incoming) {
-		info("test: event %s n_incoming=%u (expected %u)\n",
-		     bevent_str(ev),
-		     ag->n_incoming, rule->n_incoming);
-		return false;
-	}
-
-	if (UINTSET(rule->n_progress) &&
-	    ag->n_progress < rule->n_progress) {
-		info("test: event %s n_progress=%u (expected %u)\n",
-		     bevent_str(ev),
-		     ag->n_progress, rule->n_progress);
-		return false;
-	}
-
-	if (UINTSET(rule->n_established) &&
-	    ag->n_established != rule->n_established) {
-		info("test: event %s n_established=%u (expected %u)\n",
-		     bevent_str(ev),
-		     ag->n_established, rule->n_established);
-		return false;
-	}
-
-	if (UINTSET(rule->n_audio_estab) &&
-	    ag->n_audio_estab != rule->n_audio_estab) {
-		info("test: event %s n_audio_estab=%u (expected %u)\n",
-		     bevent_str(ev),
-		     ag->n_audio_estab, rule->n_audio_estab);
-		return false;
-	}
-
-	if (UINTSET(rule->n_video_estab) &&
-	    ag->n_video_estab != rule->n_video_estab) {
-		info("test: event %s n_video_estab=%u (expected %u)\n",
-		     bevent_str(ev),
-		     ag->n_video_estab, rule->n_video_estab);
-		return false;
-	}
-
-	if (UINTSET(rule->n_offer_cnt) &&
-	    ag->n_offer_cnt != rule->n_offer_cnt) {
-		info("test: event %s n_offer_cnt=%u (expected %u)\n",
-		     bevent_str(ev),
-		     ag->n_offer_cnt, rule->n_offer_cnt);
-		return false;
-	}
-
-	if (UINTSET(rule->n_answer_cnt) &&
-	    ag->n_answer_cnt != rule->n_answer_cnt) {
-		info("test: event %s n_answer_cnt=%u (expected %u)\n",
-		     bevent_str(ev),
-		     ag->n_answer_cnt, rule->n_answer_cnt);
-		return false;
-	}
-
-	if (UINTSET(rule->n_vidframe) &&
-	    ag->n_vidframe < rule->n_vidframe)
-		return false;
-
-	if (UINTSET(rule->n_auframe) &&
-	    ag->n_auframe < rule->n_auframe)
-		return false;
-
-	if (UINTSET(rule->n_audebug) &&
-	    ag->n_audebug < rule->n_audebug)
-		return false;
-
-	if (UINTSET(rule->n_rtcp) &&
-	    ag->n_rtcp < rule->n_rtcp)
-		return false;
-
-	if (UINTSET(rule->n_closed) &&
-	    ag->n_closed < rule->n_closed)
-		return false;
-
-	if (rule->aulvl != 0.0f &&
-	    (ag->aulvl < rule->aulvl || ag->aulvl >= 0.0f))
-		return false;
-
-	rule->met = true;
-out:
-
-	if (met_prev && met_next) {
-		info("canceled by %H", cancel_rule_debug, rule);
-		re_cancel();
-		cancel_rules_reset(ag->fix);
-	}
-
-	return met_next;
-}
-
-
-static void process_rules(struct agent *ag, enum bevent_ev ev, const char *prm)
-{
-	struct fixture *f = ag->fix;
-	struct le *le;
-
-	LIST_FOREACH(&f->rules, le) {
-		check_rule(le->data, true, ag, ev, prm);
-	}
-}
-
-
-static void event_handler(enum bevent_ev ev, struct bevent *event, void *arg)
-{
-	struct fixture *f = arg;
-	struct call *call2 = NULL;
-	struct agent *ag;
-	struct stream *strm = NULL;
-	char curi[256];
-	const char    *prm  = bevent_get_text(event);
-	struct call   *call = bevent_get_call(event);
-	struct ua     *ua   = bevent_get_ua(event);
-	const struct sip_msg *msg  = bevent_get_msg(event);
-	int err = 0;
-
-#if 1
-	info("test: [ %s ] event: %s (%s)\n",
-	     account_aor(ua_account(ua)), bevent_str(ev), prm);
-#endif
-
-	ASSERT_TRUE(f != NULL);
-	ASSERT_EQ(MAGIC, f->magic);
-
-	if (ev == BEVENT_CREATE)
-		return;
-
-	if (!ua)
-		ua = uag_find_msg(msg);
-
-	if (ua && ev == BEVENT_SIPSESS_CONN) {
-		err = ua_accept(ua, msg);
-		if (err) {
-			warning("test: could not accept incoming call (%m)\n",
-				err);
-			return;
-		}
-
-		bevent_stop(event);
-	}
-
-	if (ua == f->a.ua)
-		ag = &f->a;
-	else if (ua == f->b.ua)
-		ag = &f->b;
-	else if (ua == f->c.ua)
-		ag = &f->c;
-	else {
-		warning("test: could not find agent/ua\n");
-		return;
-	}
-
-	switch (ev) {
-
-	case BEVENT_CALL_REDIRECT:
-		ASSERT_STREQ("302,sip:c@127.0.0.1", prm);
-		break;
-
-	case BEVENT_CALL_INCOMING:
-		++ag->n_incoming;
-
-		switch (f->behaviour) {
-
-		case BEHAVIOUR_ANSWER:
-			err = ua_answer(ua, call, VIDMODE_ON);
-			if (err) {
-				warning("ua_answer failed (%m)\n", err);
-				goto out;
-			}
-			break;
-
-		case BEHAVIOUR_REJECT:
-			ua_hangup(ua, call, 0, 0);
-			call = NULL;
-			ag->failed = true;
-			break;
-
-		case BEHAVIOUR_REJECTF:
-			ua_hangupf(ua, call, 302, "Moved Temporarily",
-				"Contact: <sip:c@127.0.0.1>;expires=5\r\n"
-				"Diversion: <sip:a@127.0.0.1>;reason=nop\r\n"
-				"Content-Length: 0\r\n\r\n");
-			call = NULL;
-			ag->failed = true;
-			break;
-
-		case BEHAVIOUR_GET_HDRS:
-			hdrs = call_get_custom_hdrs(call);
-			err = ua_answer(ua, call, VIDMODE_ON);
-			if (err) {
-				warning("ua_answer failed (%m)\n", err);
-				goto out;
-			}
-			break;
-
-		case BEHAVIOUR_PROGRESS:
-			err = call_progress(call);
-			if (err) {
-				warning("call_progress failed (%m)\n", err);
-				goto out;
-			}
-			break;
-
-		default:
-			break;
-		}
-		break;
-
-	case BEVENT_CALL_PROGRESS:
-		++ag->n_progress;
-
-		break;
-
-	case BEVENT_CALL_ESTABLISHED:
-		++ag->n_established;
-
-		ASSERT_TRUE(str_isset(call_id(call)));
-
-		/* are both agents established? */
-		if (ag->n_established >= f->exp_estab &&
-		    ag->peer->n_established >= f->exp_estab) {
-
-			switch (f->estab_action) {
-
-			case ACTION_RECANCEL:
-				re_cancel();
-				break;
-
-			case ACTION_HANGUP_A:
-				f->a.failed = true;
-				ua_hangup(f->a.ua, NULL, 0, 0);
-				break;
-
-			case ACTION_HANGUP_B:
-				f->b.failed = true;
-				ua_hangup(f->b.ua, NULL, 0, 0);
-				break;
-
-			case ACTION_NOTHING:
-				/* Do nothing, wait */
-				break;
-
-			case ACTION_TRANSFER:
-				f->estab_action = ACTION_NOTHING;
-
-				if (f->fail_transfer)
-					f->behaviour = BEHAVIOUR_REJECT;
-
-				re_snprintf(curi, sizeof(curi),
-					    "sip:c@%J", &f->laddr_udp);
-
-				err = call_hold(ua_call(f->a.ua), true);
-				if (err)
-					goto out;
-
-				err = call_transfer(ua_call(f->a.ua), curi);
-				if (err)
-					goto out;
-				break;
-
-			case ACTION_ATT_TRANSFER:
-				re_snprintf(curi, sizeof(curi), "sip:c@%J",
-					&f->laddr_udp);
-
-				if (f->xfer) {
-					err = call_hold(ua_call(ua), true);
-					if (err)
-						goto out;
-
-					err = call_replace_transfer(f->xfer,
-						ua_call(ua));
-					if (err)
-						goto out;
-
-					f->xfer = NULL;
-					f->estab_action = ACTION_NOTHING;
-					break;
-				}
-
-				err = call_hold(ua_call(ua), true);
-				if (err)
-					goto out;
-
-				err = ua_connect(ua, NULL, NULL,
-					curi, VIDMODE_ON);
-				if (err)
-					goto out;
-
-				f->xfer = call;
-
-				break;
-			}
-		}
-		break;
-
-	case BEVENT_CALL_CLOSED:
-		++ag->n_closed;
-
-		ag->close_scode = call_scode(call);
-		mem_deref(ag->close_prm);
-		str_dup(&ag->close_prm, prm);
-
-		if (ag->close_scode)
-			ag->failed = true;
-
-		if (ag->n_closed >= f->exp_closed &&
-		    ag->peer->n_closed >= f->exp_closed) {
-
-			re_cancel();
-		}
-		break;
-
-	case BEVENT_CALL_TRANSFER:
-		++ag->n_transfer;
-
-		err = ua_call_alloc(&call2, ua, VIDMODE_ON, NULL, call,
-				call_localuri(call), true);
-		if (!err) {
-			struct pl pl;
-
-			call_set_user_data(call2, call_user_data(call));
-			pl_set_str(&pl, prm);
-
-			err = call_connect(call2, &pl);
-			if (err) {
-				warning("ua: transfer: connect error: %m\n",
-					err);
-			}
-		}
-
-		if (err) {
-			(void)call_notify_sipfrag(call, 500, "Call Error");
-			mem_deref(call2);
-		}
-		break;
-
-	case BEVENT_CALL_TRANSFER_FAILED:
-		++ag->n_transfer_fail;
-
-		call_hold(call, false);
-		if (ua == f->a.ua)
-			re_cancel();
-
-		break;
-
-	case BEVENT_CALL_REMOTE_SDP:
-		if (!str_cmp(prm, "offer"))
-			++ag->n_offer_cnt;
-		else if (!str_cmp(prm, "answer"))
-			++ag->n_answer_cnt;
-
-		break;
-
-	case BEVENT_CALL_HOLD:
-		++ag->n_hold_cnt;
-		break;
-
-	case BEVENT_CALL_RESUME:
-		++ag->n_resume_cnt;
-		break;
-
-	case BEVENT_CALL_MENC:
-		++ag->n_mediaenc;
-
-		if (strstr(prm, "audio"))
-			strm = audio_strm(call_audio(call));
-		else if (strstr(prm, "video"))
-			strm = video_strm(call_video(call));
-
-		if (strm) {
-			ASSERT_TRUE(stream_is_secure(strm));
-		}
-		break;
-
-	case BEVENT_CALL_DTMF_START:
-		ASSERT_EQ(1, str_len(prm));
-		ASSERT_EQ(dtmf_digits[ag->n_dtmf_recv], prm[0]);
-		++ag->n_dtmf_recv;
-
-		if (ag->n_dtmf_recv >= str_len(dtmf_digits)) {
-			re_cancel();
-		}
-		break;
-
-	case BEVENT_CALL_RTPESTAB:
-		++ag->n_rtpestab;
-
-		if (strstr(prm, "audio"))
-			++ag->n_audio_estab;
-		else if (strstr(prm, "video"))
-			++ag->n_video_estab;
-
-		break;
-
-	case BEVENT_CALL_RTCP:
-		++ag->n_rtcp;
-
-		break;
-
-	default:
-		break;
-	}
-
-	if (ag->failed && ag->peer->failed) {
-		info("test: re_cancel on call failed\n");
-		re_cancel();
-		return;
-	}
-
-	process_rules(ag, ev, prm);
-
- out:
-	if (err) {
-		warning("error in event-handler (%m)\n", err);
-		fixture_abort(f, err);
-	}
-}
-
-
 static int test_call_answer_priv(void)
 {
 	struct fixture fix, *f = &fix;
@@ -1336,6 +414,167 @@ static bool linenum_are_sequential(const struct ua *ua)
 }
 
 
+static void mixdetect_handler(struct auframe *af, const char *dev, void *arg)
+{
+	struct fixture *fix = arg;
+	struct agent *ag = NULL;
+	int err = 0;
+
+	err = fixture_auframe_handle(arg, af, dev, &ag);
+	if (err)
+		return;
+
+	/* only check for agent B */
+	if (ag == &fix->a)
+		return;
+
+	struct ua *ua = ag->ua;
+	int16_t *sampv = af->sampv;
+
+	/* The mixed ausrc is a square wave with double frequency.
+	 * Count zero crosses */
+	uint32_t crosses = 0;
+	int16_t last_v = sampv[0];
+	for (size_t i = 0; i < af->sampc; i++) {
+		int16_t v = sampv[i];
+#if 0
+		debug("%d ", v);
+#endif
+		if (v * last_v < 0) {
+			++crosses;
+#if 0
+			debug("%zu:%d-->%d ", i, last_v, v);
+#endif
+		}
+
+		last_v = v;
+	}
+
+	double aulvl = auframe_level(af);
+#if 0
+	debug(" crosses=%u aulevel=%lf\n", crosses, aulvl);
+#endif
+	if (!crosses && audio_rxaubuf_started(call_audio(ua_call(ua)))) {
+		debug("test: mixdetect_handler: "
+		      "detected a buffer underrun or EOS\n");
+		return;
+	}
+
+	bevent_ua_emit(BEVENT_CUSTOM, ua, crosses > 11 ? "mixed" :
+		       aulvl > -3.0 ? "original" : "low", ag->n_auframe);
+}
+
+
+static int test_call_mixausrc_priv(bool dec)
+{
+	struct fixture fix, *f = &fix;
+	struct cancel_rule *cr;
+	struct ausrc *ausrc = NULL;
+	struct auplay *auplay = NULL;
+	const char *dp = test_datapath();
+	char *prm = NULL;
+	int err = 0;
+
+	err = re_sdprintf(&prm, ";ptime=10"
+			  ";audio_source=aufile,%s/wav/square_500Hz_0.8.wav"
+			  ";audio_player=mock-auplay,a", dp);
+	TEST_ERR(err);
+	fixture_init_prm(f, prm);
+	prm = mem_deref(prm);
+	mem_deref(f->b.ua);
+
+	err = re_sdprintf(&prm, "B <sip:b@127.0.0.1>;regint=0;ptime=10"
+			  ";audio_source=aufile,%s/wav/square_500Hz_0.8.wav"
+			  ";audio_player=mock-auplay,b", dp);
+	TEST_ERR(err);
+	err = ua_alloc(&f->b.ua, prm);
+	prm = mem_deref(prm);
+	TEST_ERR(err);
+
+	conf_config()->avt.rtp_stats = true;
+	conf_config()->avt.audio.jbuf_del.min = 10;
+
+	cancel_rule_new(BEVENT_CUSTOM, f->b.ua, 1, 0, -1);
+	cr->prm = "auframe";
+	cr->n_auframe = 3;
+
+	err = module_load(".", "aufile");
+	TEST_ERR(err);
+	err = module_load(".", "mixausrc");
+	TEST_ERR(err);
+
+	err = mock_auplay_register(&auplay, baresip_auplayl(),
+				   mixdetect_handler, f);
+	TEST_ERR(err);
+
+	f->behaviour = BEHAVIOUR_ANSWER;
+	f->estab_action = ACTION_NOTHING;
+
+	/* Make a call from A to B */
+	err = ua_connect(f->a.ua, 0, NULL,f->buri, VIDMODE_OFF);
+	TEST_ERR(err);
+
+	/* run main-loop with timeout, wait for events */
+	err = re_main_timeout(5000);
+	TEST_ERR(err);
+	TEST_ERR(fix.err);
+
+	cancel_rule_pop();
+	cancel_rule_new(BEVENT_CUSTOM, f->b.ua, 1, 0, -1);
+	cr->prm = "mixed";
+
+	err = re_sdprintf(&prm,	"mixausrc_%s_start aufile "
+				"%s/wav/square_1kHz_0.4.wav 25 100 cname=%s "
+				"fadetime=10",
+				dec ? "dec" : "enc", dp, dec ? "b@" : "a@");
+	TEST_ERR(err);
+	err = fixture_delayed_command(f, 0, prm);
+	prm = mem_deref(prm);
+	TEST_ERR(err);
+	err = re_main_timeout(1000);
+	TEST_ERR(err);
+	TEST_ERR(fix.err);
+
+	cancel_rule_pop();
+	cancel_rule_new(BEVENT_CUSTOM, f->b.ua, 1, 0, 1);
+	cr->prm = "original";
+	err = fixture_delayed_command(f, 0, dec ?
+				      "mixausrc_dec_stop cname=b@" :
+				      "mixausrc_enc_stop cname=a@");
+	TEST_ERR(err);
+
+	err = re_main_timeout(1000);
+	TEST_ERR(err);
+	TEST_ERR(fix.err);
+
+ out:
+	mem_deref(prm);
+	conf_config()->avt.audio.jbuf_del.min = 100;
+	fixture_close(f);
+	mem_deref(ausrc);
+	mem_deref(auplay);
+	module_unload("mixausrc");
+	module_unload("aufile");
+	if (fix.err && !err)
+		err = fix.err;
+
+	return err;
+}
+
+
+int test_call_mixausrc(void)
+{
+	int err;
+	err = test_call_mixausrc_priv(false);
+	TEST_ERR(err);
+	err = test_call_mixausrc_priv(true);
+	TEST_ERR(err);
+
+out:
+	return err;
+}
+
+
 int test_call_multiple(void)
 {
 	struct fixture fix, *f = &fix;
@@ -1477,11 +716,11 @@ int test_call_max(void)
 int test_call_dtmf(void)
 {
 	struct fixture fix, *f = &fix;
-	size_t i, n = str_len(dtmf_digits);
 	int err = 0;
 
 	/* Use a low packet time, so the test completes quickly */
 	fixture_init_prm(f, ";ptime=1");
+	f->dtmf_digits = "1234";
 
 	/* audio-source is needed for dtmf/telev to work */
 	err = module_load(".", "ausine");
@@ -1499,18 +738,29 @@ int test_call_dtmf(void)
 	TEST_ERR(fix.err);
 
 	/* send some DTMF digits from A to B .. */
-	for (i=0; i<n; i++) {
-		err  = call_send_digit(ua_call(f->a.ua), dtmf_digits[i]);
+	size_t n = str_len(f->dtmf_digits);
+	for (size_t i=0; i<n; i++) {
+		err  = call_send_digit(ua_call(f->a.ua), f->dtmf_digits[i]);
 		TEST_ERR(err);
 	}
 
+	err = call_send_digit(ua_call(f->a.ua), KEYCODE_REL);
+	TEST_ERR(err);
+
+	struct audio *audio = call_audio(ua_call(f->a.ua));
+	ASSERT_TRUE(audio != NULL);
+	ASSERT_TRUE(!audio_txtelev_empty(audio));
+
 	/* run main-loop with timeout, wait for events */
 	err = re_main_timeout(5000);
 	TEST_ERR(err);
 	TEST_ERR(fix.err);
 
 	ASSERT_EQ(0, fix.a.n_dtmf_recv);
-	ASSERT_EQ(n, fix.b.n_dtmf_recv);
+	ASSERT_EQ((unsigned) n, fix.b.n_dtmf_recv);
+	audio = call_audio(ua_call(f->a.ua));
+	ASSERT_TRUE(audio != NULL);
+	ASSERT_TRUE(audio_txtelev_empty(audio));
 
  out:
 	fixture_close(f);
@@ -1827,40 +1077,8 @@ int test_call_100rel_video(void)
 static void auframe_handler(struct auframe *af, const char *dev, void *arg)
 {
 	struct fixture *fix = arg;
-	struct agent *ag = NULL;
-	struct ua *ua;
-	int err = 0;
-	(void)af;
 
-	ASSERT_EQ(MAGIC, fix->magic);
-
-	if (!str_cmp(dev, "a")) {
-		ag = &fix->a;
-	}
-	else if (!str_cmp(dev, "b")) {
-		ag = &fix->b;
-	}
-	else {
-		warning("test: received audio frame - agent unclear\n");
-		return;
-	}
-
-	ua = ag->ua;
-	/* Does auframe come from the decoder ? */
-	if (!audio_rxaubuf_started(call_audio(ua_call(ua)))) {
-		debug("test: [%s] no audio received from decoder yet\n",
-		      account_aor(ua_account(ua)));
-		return;
-	}
-
-	++ag->n_auframe;
-	(void)audio_level_get(call_audio(ua_call(ua)), &ag->aulvl);
-
-	bevent_ua_emit(BEVENT_CUSTOM, ua, "auframe %u", ag->n_auframe);
-
- out:
-	if (err)
-		fixture_abort(fix, err);
+	fixture_auframe_handle(fix, af, dev, NULL);
 }
 
 
@@ -2342,9 +1560,9 @@ int test_call_custom_headers(void)
 	/* run main-loop with timeout, wait for events */
 	err = re_main_timeout(5000);
 
-	if (!list_isempty(hdrs)) {
+	if (!list_isempty(f->hdrs)) {
 		struct le *le;
-		for (le = list_head(hdrs); le; le = le->next) {
+		for (le = list_head(f->hdrs); le; le = le->next) {
 		    struct sip_hdr *hdr = le->data;
 		    if (pl_strcasecmp(&hdr->name, "X-CALL_ID") == 0) {
 		        char buf[20];
@@ -3562,6 +2780,10 @@ int test_call_sni(void)
 	err |= dns_server_add_a(dns_srv, "retest.server.org", IP_127_0_0_1);
 	err |= dns_server_add_a(dns_srv, "retest.unknown.org", IP_127_0_0_1);
 	err |= dnsc_alloc(&dnsc, NULL, &dns_srv->addr, 1);
+	TEST_ERR(err);
+
+	dnsc_cache_max(dnsc, 0);
+
 	err |= net_set_dnsc(baresip_network(), dnsc);
 	TEST_ERR(err);
 
