@@ -34,6 +34,7 @@ static const struct test tests[] = {
 	TEST(test_call_max),
 	TEST(test_call_mediaenc),
 	TEST(test_call_medianat),
+	TEST(test_call_mixausrc),
 	TEST(test_call_multiple),
 	TEST(test_call_progress),
 	TEST(test_call_reject),
@@ -61,6 +62,8 @@ static const struct test tests[] = {
 #endif
 	TEST(test_cmd),
 	TEST(test_cmd_long),
+	TEST(test_cparam_call_decode),
+	TEST(test_cparam_ua_decode),
 	TEST(test_contact),
 	TEST(test_bevent_register),
 	TEST(test_jbuf),
@@ -69,6 +72,7 @@ static const struct test tests[] = {
 	TEST(test_jbuf_gnack),
 	TEST(test_message),
 	TEST(test_network),
+	TEST(test_peerconn),
 	TEST(test_play),
 	TEST(test_stunuri),
 	TEST(test_ua_alloc),
